function [ NewAxisZ, RFfield, dRFfield ] = EqColdCavityField( kSquareParallel, AxisZ, f0initial, df0initial, ICalkaP )
% Solves RF field equation using cold cavity approach - imaginary digits
% kSquareParallel - k2z - square of wavevector parallel to z
% f0initial - initial value of f (RFfield)
% df0initial - initial derivative of f function
% AsixZ - axis Z [mm]
% dRFfield - rf field derivative

f0initial =[real(f0initial);imag(f0initial);real(df0initial);imag(df0initial)]; % way to pass imaginary number to solver

[nn,dfi]=ode45(@(nn,dfi) OdeEqColdCavityField(nn,dfi,AxisZ,kSquareParallel, ICalkaP),AxisZ, f0initial, ICalkaP);

NewAxisZ = nn;
RFfield = dfi(:,1)+1i*dfi(:,2); % skladamy liczbe zespolona spowrotem
dRFfield = dfi(:,3)+1i*dfi(:,4);
end


function dfi = OdeEqColdCavityField( z, fi, nz, k2z, ICalkaP )
% Calculate cold cavity field one step - takes imaginary numbers as
% parameters
%  
% fi = [value of field in step i; value of field derivative in step i]

if nargin<5
    PrawaStrona = 0;  % I*CalkaP
else
    PrawaStrona = interp1(nz,ICalkaP,z);
end

y(1) = fi(1)+1i*fi(2); % first equation in system
y(2) = fi(3)+1i*fi(4); % second equation in system

k2 = interp1(nz,k2z,z);

df = zeros(2,1);
df(1) = y(2); % derivative of first equation in system
df(2) = PrawaStrona - k2.*y(1); % derivative of second equation in system


dfi = [real(df(1));imag(df(1));real(df(2));imag(df(2))];

end
