function [ NewAxisZ, RFfield, Foptimal, Qoptimal, BoundaryConditionErrorOptimal ] = EqColdCavityFieldOptimization( SpeedOfLight, CharacteristicValue, ResonatorRadius, AxisZ, Fspan, Qspan, ICalkaP )
% Solves RF field equation using cold cavity approach with minimalization
% kSquareParallel - k2z - wektor falowy wzdluz osi z
% f0initial - initial value of f (RFfield)
% df0initial - initial value of f derivative
% AsixZ - Z axis[mm]
% Fspan - Frequency search range
% Qspan - diffractive quality factor search range

if nargin < 7
    ICalkaP = zeros(size(AxisZ));
end

fstart = (Fspan(1)+Fspan(end))/2;
Qstart = (Qspan(1)+Qspan(end))/2;

options = psoptimset('UseParallel',true);
start = [fstart,Qstart];
objfun = @(x)cav_solver(x,CharacteristicValue,ResonatorRadius,SpeedOfLight,AxisZ, ICalkaP);
[x,fval,~,~] = patternsearch(objfun, start, [],[],[],[], [Fspan(1),Qspan(1)],[Fspan(end),Qspan(end)],[],options);

Qoptimal=x(2);
Foptimal=x(1);


BoundaryConditionErrorOptimal = fval;
% [(k/c)^2-U(z)]
kSquareParallel = (1+1i/Qoptimal)*(2*pi*Foptimal/SpeedOfLight)^2-(CharacteristicValue./ResonatorRadius).^2;    

f0initial = exp(1i*vsqrt(kSquareParallel(1)));
df0initial = 1i*vsqrt(kSquareParallel(1))*f0initial;
[ NewAxisZ, RFfield, ~ ] = EqColdCavityField( kSquareParallel, AxisZ, f0initial, df0initial, ICalkaP );

end


function BoundaryConditionError = cav_solver(x,CharacteristicValue,ResonatorRadius,SpeedOfLight,AxisZ, ICalkaP)
        F=x(1);Q=x(2); % F and Q that we check
    
        kSquareParallel = (1+1i/Q)*(2*pi*F/SpeedOfLight)^2-(CharacteristicValue./ResonatorRadius).^2;
        f0initial = exp(1i*vsqrt(kSquareParallel(1)));
        df0initial = 1i*vsqrt(kSquareParallel(1))*f0initial;  % Edgcomb book page 49
    
        [ ~, RFfield, dRFfield ] = EqColdCavityField( kSquareParallel, AxisZ, f0initial, df0initial, ICalkaP );

        fout = RFfield(end);
        dfout = dRFfield(end);
      
        % Here we check optimalization condition |Rout|
        Rout = abs((dfout+1i*vsqrt(kSquareParallel(end))*fout)/(dfout-1i*vsqrt(kSquareParallel(end))*fout));
        BoundaryConditionError = Rout;
end
