function [ NewAxisZ, RFfield, Foptimal, Qoptimal, BoundaryConditionErrorOptimal ] = EqColdCavityFullReview( SpeedOfLight, CharacteristicValue, ResonatorRadius, AxisZ, Fspan, Qspan, ICalkaP )
% Solves RF field equation using cold cavity approach with minimalization
% kSquareParallel - k2z - wektor falowy wzdluz osi z
% f0initial - initial value of f (RFfield)
% df0initial - initial value of f derivative
% AsixZ - Z axis[mm]
% Fspan - Frequency search range
% Qspan - diffractive quality factor search range

if nargin < 7
    ICalkaP = zeros(size(AxisZ));
end

%fstart = (Fspan(1)+Fspan(end))/2;
%Qstart = (Qspan(1)+Qspan(end))/2;

%options = psoptimset('UseParallel',true);
%start = [fstart,Qstart];



%objfun = @(x)cav_solver(x,CharacteristicValue,ResonatorRadius,SpeedOfLight,AxisZ, ICalkaP);
%[x,fval,~,~] = patternsearch(objfun, start, [],[],[],[], [Fspan(1),Qspan(1)],[Fspan(end),Qspan(end)],[],options);

fval = zeros(length(Fspan), length(Qspan));
fidxMin = 1; QidxMin = 1;
fvalmin = cav_solver([Fspan(fidxMin), Qspan(QidxMin)],CharacteristicValue,ResonatorRadius,SpeedOfLight,AxisZ, ICalkaP); % initialize with first value
Qlen = length(Qspan); % avoid matlab parfor classification problem
parfor fidx = 1:length(Fspan)
    for Qidx = 1:Qlen
         fval(fidx, Qidx) = cav_solver([Fspan(fidx), Qspan(Qidx)],CharacteristicValue,ResonatorRadius,SpeedOfLight,AxisZ, ICalkaP);
    end
end

% Choose best solution
for fidx = 1:length(Fspan)
    for Qidx = 1:length(Qspan)
         if(fval(fidx, Qidx) < fvalmin)
            fvalmin =  fval(fidx, Qidx);
            fidxMin = fidx;  % store Fspan vector index where fval minimum was found
            QidxMin = Qidx;
         end
    end
end

Qoptimal=Qspan(QidxMin);
Foptimal=Fspan(fidxMin);


BoundaryConditionErrorOptimal = fvalmin;
figHandle = figure('units','centimeters','position',[10,10,30,25]);
[X,Y] = meshgrid(Fspan, Qspan);
Z = fval';
h = surf(X, Y, Z);
%set(h,'edgecolor','none');  % uncoment if plot looks black
title('|R|');
zlabel("Boundary condition error")
ylabel("Q_{diff} [a.u.]")
xlabel("Frequency [GHz]")
set(gca,'XColor', 'none','YColor','none')
set(gca,'XColor', 'black','YColor','black') 
saveas(figHandle,sprintf('absR.png'), 'png'); % save fig to file

% [(k/c)^2-U(z)]
kSquareParallel = (1+1i/Qoptimal)*(2*pi*Foptimal/SpeedOfLight)^2-(CharacteristicValue./ResonatorRadius).^2;    

f0initial = exp(1i*vsqrt(kSquareParallel(1)));
df0initial = 1i*vsqrt(kSquareParallel(1))*f0initial;
[ NewAxisZ, RFfield, ~ ] = EqColdCavityField( kSquareParallel, AxisZ, f0initial, df0initial, ICalkaP );

end


function BoundaryConditionError = cav_solver(x,CharacteristicValue,ResonatorRadius,SpeedOfLight,AxisZ, ICalkaP)
        F=x(1);Q=x(2); % F and Q that we check
    
        kSquareParallel = (1+1i/Q)*(2*pi*F/SpeedOfLight)^2-(CharacteristicValue./ResonatorRadius).^2;
        f0initial = exp(1i*vsqrt(kSquareParallel(1)));
        df0initial = 1i*vsqrt(kSquareParallel(1))*f0initial;  % Edgcomb book page 49
    
        [ ~, RFfield, dRFfield ] = EqColdCavityField( kSquareParallel, AxisZ, f0initial, df0initial, ICalkaP );

        fout = RFfield(end);
        dfout = dRFfield(end);
      
        % Here we check optimalization condition |Rout|
        Rout = abs((dfout+1i*vsqrt(kSquareParallel(end))*fout)/(dfout-1i*vsqrt(kSquareParallel(end))*fout));
        
        BoundaryConditionError = Rout;
end
