function [ OUTPUT ] = OptymizeAlphaFunction( INPUT , C , DoPlots)

% Wejsciem jest struktura SC.I - self consistent input, wartosc B0 jest
% optymalizowana


SC.I = INPUT;
% C - physical constants
%% Przemiatamy po polu magnetycznym - szukamy optymalnej wartości
if isfield(SC.I, 'Alpha')
        disp('UWAGA: Podane Alpha zignorowane, ta funkcja sprawdza wszystkie Alpha!')
end

%AlphaSpan = (1.2:0.01:1.5);
AlphaSpan = (0.5:0.1:1.5);

for m = 1:length(AlphaSpan)
    SC.I.Alpha = AlphaSpan(m);
    %SC.O.SOLUTIONS{m} = SelfConsistentApproachFunction(SC.I, C);
    SC.O.SOLUTIONSforALPHA{m} = OptymizeMagneticFieldFunction(SC.I, C, 0);
end
% Szukanie najlepszego wyniku
for m = 1:length(AlphaSpan)
    SC.O.GyrotronEfficiencyAll(m) = SC.O.SOLUTIONSforALPHA{m}.GyrotronEfficiency;
    SC.O.AlphaAll(m) = SC.O.SOLUTIONSforALPHA{m}.Alpha;
end

[SC.O.MaxEfficiencyValue, SC.O.MaxEfficiencyIndex] = max(SC.O.GyrotronEfficiencyAll);

% Best solution parameters
SC.O.MagneticFieldB = SC.O.SOLUTIONSforALPHA{SC.O.MaxEfficiencyIndex}.MagneticFieldB;
SC.O.F = SC.O.SOLUTIONSforALPHA{SC.O.MaxEfficiencyIndex}.F;
SC.O.mnSign = SC.O.SOLUTIONSforALPHA{SC.O.MaxEfficiencyIndex}.mnSign;
SC.O.AxisZeta = SC.O.SOLUTIONSforALPHA{SC.O.MaxEfficiencyIndex}.AxisZeta;
SC.O.GyrotronEfficiency = SC.O.SOLUTIONSforALPHA{SC.O.MaxEfficiencyIndex}.GyrotronEfficiency;
SC.O.AllEletronMomentum = SC.O.SOLUTIONSforALPHA{SC.O.MaxEfficiencyIndex}.AllEletronMomentum;
SC.O.NumberOfElectrons = SC.O.SOLUTIONSforALPHA{SC.O.MaxEfficiencyIndex}.NumberOfElectrons;
SC.O.Alpha = SC.O.SOLUTIONSforALPHA{SC.O.MaxEfficiencyIndex}.Alpha;
SC.O.Ucathode = SC.O.SOLUTIONSforALPHA{SC.O.MaxEfficiencyIndex}.Ucathode;
SC.O.ElectronBeamRadius = SC.O.SOLUTIONSforALPHA{SC.O.MaxEfficiencyIndex}.ElectronBeamRadius;
SC.O.GyrotronOutputPower = SC.O.SOLUTIONSforALPHA{SC.O.MaxEfficiencyIndex}.GyrotronOutputPower;
SC.O.Icathode = SC.O.SOLUTIONSforALPHA{SC.O.MaxEfficiencyIndex}.Icathode;
SC.O.FrequencyMismatch = SC.O.SOLUTIONSforALPHA{SC.O.MaxEfficiencyIndex}.FrequencyMismatch;



if DoPlots == 1
%% Wykres Momentow Pędu najlepszego rozwiązania
figure();
plot(SC.O.AxisZeta,abs(SC.O.AllEletronMomentum),SC.O.AxisZeta,abs(SC.I.RFfieldRaw/max(SC.I.RFfieldRaw)),'r.-'); ylabel('|p(\zeta)|, |f(\zeta)|');xlabel('\zeta');
set(gcf(),'Name','p(z)');
% figure();
% plot(180*Theta/pi,abs(SC.O.AllEletronMomentum(end,:)),'*-',180*Theta/pi,ones(size(Theta)),'r-'); ylabel('\int{|p(\theta,\zeta)|^2 d\theta}');xlabel('\theta');
% set(gcf(),'Name','p(z_out)');

%% Wykres Alpha vs magnetic field

AlphaLength = length(AlphaSpan);
MagneticFieldLength = length(SC.O.SOLUTIONSforALPHA{1}.GyrotronEfficiencyAll);

WykresEfficiency = zeros(AlphaLength, MagneticFieldLength);
WykresAlphaAxis = AlphaSpan;

WykresMagneticFieldAxis = SC.O.SOLUTIONSforALPHA{1}.MagneticFieldBAll;
for AlphaIdx = 1:AlphaLength
    
    WykresEfficiency(AlphaIdx,:) = SC.O.SOLUTIONSforALPHA{AlphaIdx}.GyrotronEfficiencyAll;
end

figure(); set(gcf(),'Name','Efficiency Alpha vs Magnetic Field');
surf(WykresMagneticFieldAxis, WykresAlphaAxis, WykresEfficiency);
zlabel('Efficiency'); ylabel('Alpha'); xlabel('Magnetic Field'); 
end

% Delete what takes space and is not required
for m = 1:length(AlphaSpan)
    SC.O.SOLUTIONSforALPHA{m}.AllEletronMomentum = [];
end
%%
OUTPUT = SC.O;  % Wyjscie

end