function [ OUTPUT ] = OptymizeMagneticFieldFunction( INPUT , C , DoPlots)

% Wejsciem jest struktura SC.I - self consistent input, wartosc B0 jest
% optymalizowana
%DoPlots = 0;  czy rosowac wykresy

SC.I = INPUT;
% C - physical constants
%% Przemiatamy po polu magnetycznym - szukamy optymalnej wartości
if isfield(SC.I, 'MagneticFieldB')
        disp('UWAGA: Podane pole magnetyczne zignorowane, ta funkcja optymalizuje pole magnetyczne!')
end

SC.OmegaResonator = 2*pi*SC.I.Frequency;

SC.GammaElectron = 1 + SC.I.Ucathode/((C.ElectronMass*(C.SpeedOfLight)^2)/C.ElectronCharge);

%MagneticFieldSpan = (0.9999:0.0001:1)*SC.GammaElectron*SC.I.Frequency*(2*pi*C.ElectronMass/C.ElectronCharge)/SC.I.HarmonicNumber;
%MagneticFieldSpan = (0.95:0.0001:1)*SC.GammaElectron*SC.I.Frequency*(2*pi*C.ElectronMass/C.ElectronCharge)/SC.I.HarmonicNumber;
MagneticFieldEstimate = SC.GammaElectron*SC.I.Frequency*(2*pi*C.ElectronMass/C.ElectronCharge)/SC.I.HarmonicNumber;
MagneticFieldSpan = (0.90*MagneticFieldEstimate:0.0001:1*MagneticFieldEstimate);

dane=sprintf('=============================== Optymizing magnetic field ===============================');disp(dane);
dane=sprintf('Bmin = %.3f, Bmax = %.3f [T]', min(MagneticFieldSpan), max(MagneticFieldSpan));disp(dane);

% Przygotuj dane dla parfora
for m = 1:length(MagneticFieldSpan)
    SC.I.MagneticFieldB = MagneticFieldSpan(m);
    ParforIn(m) = SC.I;
end

parfor m = 1:length(MagneticFieldSpan)
    ParforOut(m) = SelfConsistentApproachFunction(ParforIn(m), C, 0, 0);
end
%for m = 1:length(MagneticFieldSpan)
%     SC.O.SOLUTIONSforB{m} = SelfConsistentApproachFunction(ParforIn(m), C, 0, 0);
%end
% Odbierz dane z parfora
for m = 1:length(MagneticFieldSpan)
    SC.O.SOLUTIONSforB{m} = ParforOut(m);
end
% Szukanie najlepszego wyniku
for m = 1:length(MagneticFieldSpan)
    SC.O.GyrotronEfficiencyAll(m) = SC.O.SOLUTIONSforB{m}.GyrotronEfficiency;
    SC.O.MagneticFieldBAll(m) = SC.O.SOLUTIONSforB{m}.MagneticFieldB;
end

[SC.O.MaxEfficiencyValue, SC.O.MaxEfficiencyIndex] = max(SC.O.GyrotronEfficiencyAll);

% Best solution parameters
SC.O.MagneticFieldB = SC.O.SOLUTIONSforB{SC.O.MaxEfficiencyIndex}.MagneticFieldB;
SC.O.F = SC.O.SOLUTIONSforB{SC.O.MaxEfficiencyIndex}.F;
SC.O.mnSign = SC.O.SOLUTIONSforB{SC.O.MaxEfficiencyIndex}.mnSign;
SC.O.AxisZeta = SC.O.SOLUTIONSforB{SC.O.MaxEfficiencyIndex}.AxisZeta;
SC.O.GyrotronEfficiency = SC.O.SOLUTIONSforB{SC.O.MaxEfficiencyIndex}.GyrotronEfficiency;
SC.O.AllEletronMomentum = SC.O.SOLUTIONSforB{SC.O.MaxEfficiencyIndex}.AllEletronMomentum;
SC.O.NumberOfElectrons = SC.O.SOLUTIONSforB{SC.O.MaxEfficiencyIndex}.NumberOfElectrons;
SC.O.Alpha = SC.O.SOLUTIONSforB{SC.O.MaxEfficiencyIndex}.Alpha;
SC.O.Ucathode = SC.O.SOLUTIONSforB{SC.O.MaxEfficiencyIndex}.Ucathode;
SC.O.ElectronBeamRadius = SC.O.SOLUTIONSforB{SC.O.MaxEfficiencyIndex}.ElectronBeamRadius;
SC.O.GyrotronOutputPower = SC.O.SOLUTIONSforB{SC.O.MaxEfficiencyIndex}.GyrotronOutputPower;
SC.O.Icathode = SC.O.SOLUTIONSforB{SC.O.MaxEfficiencyIndex}.Icathode;
SC.O.FrequencyMismatch = SC.O.SOLUTIONSforB{SC.O.MaxEfficiencyIndex}.FrequencyMismatch;
SC.O.LarmorRadius = SC.O.SOLUTIONSforB{SC.O.MaxEfficiencyIndex}.LarmorRadius;

% Delete what takes space and is not required
for m = 1:length(MagneticFieldSpan)
    %SC.O.SOLUTIONSforB{m}.AllEletronMomentum = [];
    SC.O.SOLUTIONSforB{m} = [];
end

dane=sprintf('=============================== Best magnetic field ===============================');disp(dane);
SelfConsistentApproachFunction(ParforIn(SC.O.MaxEfficiencyIndex), C, 0, 0); % wyswietlam najlepszy wynik

if DoPlots == 1
figure();
SC.RFfieldNormalized = SC.I.RFfieldRaw/max(abs(SC.I.RFfieldRaw));
Theta = 2*pi*(0:SC.I.NumberOfElectrons-1)/SC.I.NumberOfElectrons;
plot(SC.O.AxisZeta,abs(SC.O.AllEletronMomentum),SC.O.AxisZeta,abs(SC.RFfieldNormalized),'r.-'); ylabel('|p(\zeta)|, |f(\zeta)|');xlabel('\zeta');
set(gcf(),'Name','p(z)');
figure();
plot(180*Theta/pi,abs(SC.O.AllEletronMomentum(end,:)),'*-',180*Theta/pi,ones(size(Theta)),'r-'); ylabel('\int{|p(\theta,\zeta)|^2 d\theta}');xlabel('\theta');
set(gcf(),'Name','p(z_out)');
end

OUTPUT = SC.O;
end