clear all; close all;

R0 = 7.5; M=1; K=1; 
%Rc = abs(M)*R0/enju;
%rr = (0.001*Rc:.0001:R0);
enju = bessel_deriv_zeros(M,K);
kperp = enju/R0;
rr = (R0*0.01:.0001:R0);
x = (0:.0001:4*enju);

% Bessel function - first kind M-th order
hold on;
temp_plus=besselj(M,x);
plot(x,temp_plus,'.-'); LegendDesc = {'J_{M}(r)'};
if(M>=1)
    temp_minus=besselj(M-1,x);
    plot(x,temp_minus,'.-'); LegendDesc{end+1,:} = 'J_{M-1}(r)';
end
temp_plus=besselj(M+1,x);
plot(x,temp_plus,'.-'); LegendDesc{end+1,:} = 'J_{M+1}(r)';
if(M>=2)
    temp_minus=besselj(M-2,x);
    plot(x,temp_minus,'.-'); LegendDesc{end+1,:} = 'J_{M-2}(r)';
end
temp_plus=besselj(M+2,x);
plot(x,temp_plus,'.-'); LegendDesc{end+1,:} = 'J_{M+2}(r)';
if(M>=3)
    temp_minus=besselj(M-3,x);
    plot(x,temp_minus,'.-'); LegendDesc{end+1,:} = 'J_{M-3}(r)';
end
temp_plus=besselj(M+3,x);
plot(x,temp_plus,'.-'); LegendDesc{end+1,:} = 'J_{M+3}(r)';
legend(LegendDesc);
title('Bessel function of first kind M-th order.');
%set(gca,'YColor','none');
set(gca,'YTick', []);
hold off;

% Bessel function derivative - first kind M-th order

figure();
hold on;
temp_plus=diff(besselj(M,x));
xx = x(1:end-1);
plot(xx,temp_plus,'.-'); LegendDesc = {'J_{M}(r)'};
if(M>=1)
    temp_minus=diff(besselj(M-1,x));
    plot(xx,temp_minus,'.-'); LegendDesc{end+1,:} = 'J_{M-1}(r)';
end
temp_plus=diff(besselj(M+1,x));
plot(xx,temp_plus,'.-'); LegendDesc{end+1,:} = 'J_{M+1}(r)';
if(M>=2)
    temp_minus=diff(besselj(M-2,x));
    plot(xx,temp_minus,'.-'); LegendDesc{end+1,:} = 'J_{M-2}(r)';
end
temp_plus=diff(besselj(M+2,x));
plot(xx,temp_plus,'.-'); LegendDesc{end+1,:} = 'J_{M+2}(r)';
if(M>=3)
    temp_minus=diff(besselj(M-3,x));
    plot(xx,temp_minus,'.-'); LegendDesc{end+1,:} = 'J_{M-3}(r)';
end
temp_plus=diff(besselj(M+3,x));
plot(xx,temp_plus,'.-'); LegendDesc{end+1,:} = 'J_{M+3}(r)';
legend(LegendDesc);
title('Bessel function of first kind M-th order derivative.');
%set(gca,'YColor','none');
set(gca,'YTick', []);
hold off;




%% Radial EM power distribution for first kind M-th order
figure();
hold on;

% Promien elektronow
%ElectronBeamRadiusPlus = bessel_deriv_zeros(SC.I.ModM+SC.I.HarmonicNumber,1)*(C.SpeedOfLight/SC.I.Frequency)/(2*pi);
%ElectronBeamRadiusMinus = bessel_deriv_zeros(SC.I.ModM-SC.I.HarmonicNumber,1)*(C.SpeedOfLight/SC.I.Frequency)/(2*pi);

% zero power at the wall - E=0 at metalic wall, from boundary condition
rrr = rr(1:end-1);
temp_plus=diff(besselj(M,bessel_deriv_zeros(M,K)*rr/R0)).^2;
plot(rrr,temp_plus/max(temp_plus),'.-'); LegendDesc = {'J_{M}(r)'};
if(M>=1)
    temp_minus=diff(besselj(M-1,bessel_deriv_zeros(M-1,K)*rr/R0)).^2;
    plot(rrr,temp_minus/max(temp_minus),'.-'); LegendDesc{end+1,:} = 'J_{M-1}(r)';
end
temp_plus=diff(besselj(M+1,bessel_deriv_zeros(M+1,K)*rr/R0)).^2;
plot(rrr,temp_plus/max(temp_plus),'.-'); LegendDesc{end+1,:} = 'J_{M+1}(r)';
if(M>=2)
    temp_minus=diff(besselj(M-2,bessel_deriv_zeros(M-2,K)*rr/R0)).^2;
    plot(rrr,temp_minus/max(temp_minus),'.-'); LegendDesc{end+1,:} = 'J_{M-2}(r)';
end
temp_plus=diff(besselj(M+2,bessel_deriv_zeros(M+2,K)*rr/R0)).^2;
plot(rrr,temp_plus/max(temp_plus),'.-'); LegendDesc{end+1,:} = 'J_{M+2}(r)';
if(M>=3)
    temp_minus=diff(besselj(M-3,bessel_deriv_zeros(M-3,K)*rr/R0)).^2;
    plot(rrr,temp_minus/max(temp_minus),'.-'); LegendDesc{end+1,:} = 'J_{M-3}(r)';
end
temp_plus=diff(besselj(M+3,bessel_deriv_zeros(M+3,K)*rr/R0)).^2;
plot(rrr,temp_plus/max(temp_plus),'.-'); LegendDesc{end+1,:} = 'J_{M+3}(r)';
legend(LegendDesc);
title('E-field power profile.');
ylabel("Normalized power [a.u.]")
xlabel("Radius [mm]")
hold off;


%% Resonator cross-section
B0 = 1;
freq = 24.5; %[GHz]
z = 0;
t = 2*pi/1;

Theta = 0:2*pi/128:2*pi;
%fi = pi/2;
C = PhysicalConstants;

omega = 2*pi*freq;
enju = bessel_deriv_zeros(M,K); % characteristic value
kz=sqrt((omega/C.SpeedOfLight)^2-(enju/R0)^2);
hold on;
J = besselj(M,enju*rr/R0);
Jdiff = diff(J);

rr = (0.1:.1:R0);

[RR,TH] = ndgrid(rr,Theta);

Z = zeros(size(RR,1), size(RR,2));
J = besselj(M,enju.*RR/R0);

ZErr = zeros(size(RR,1) , size(RR,2));
ZEth = zeros(size(RR,1) , size(RR,2));
for row=1:size(RR,1) 

    for col=1:size(RR,2)
       r = RR(:, col);
       th = TH(1, col);
       ZErr(:,col) = -B0*M*omega*R0*(1/enju)^2*(R0./r) .* besselj(M,enju*r/R0) * sin(M*th + kz*z - omega*t);
      
       tmp = B0*omega*R0*(1/enju)*R0*diff(besselj(M,enju*r/R0)) * cos(M*th + kz*z - omega*t);
       tmp = [tmp; tmp(end)]; % adding one element to match matrixes length
       ZEth(:,col) = tmp;
    
    end
end

figure();
ZE = sqrt(ZEth.^2 + ZErr.^2);
h = surf(RR.*cos(TH), RR.*sin(TH),ZEth.^2); % Matlab plots in kartesian space
%set(h,'edgecolor','none');
title('E-field angular component power.');
zlabel("Power [a.u]")
%ylabel("Radius [mm]")
%xlabel("Radius [mm]")
set(gca,'XColor', 'none','YColor','none')

figure();
h = surf(RR.*cos(TH), RR.*sin(TH),ZErr.^2);
%set(h,'edgecolor','none');
title('E-field radial component power.');
zlabel("Power [a.u]")
%ylabel("Radius [mm]")
%xlabel("Radius [mm]")
set(gca,'XColor', 'none','YColor','none')

figure();
colormap(jet);
h = surf(RR.*cos(TH), RR.*sin(TH),ZE.^2./max(ZE.^2,[],'all')); % Normalized to one
%set(h,'edgecolor','none');
title('E-field total power distribution.');
zlabel("Power [a.u]")
set(gca,'XColor', 'none','YColor','none')
%set(gca,'visible','off')
%set(h.Ticks, 'xTickLabel', '');
%ylabel("Radius [mm]")
%xlabel("Radius [mm]")


figure();
colormap(jet);
h = contourf(RR.*cos(TH), RR.*sin(TH),ZE.^2./max(ZE.^2,[],'all')); % Normalized to one

colorbar;
set(gca,'visible','off')

