clear all; close all;

R0 = 7.5; M=31; K=11; 
%Rc = abs(M)*R0/enju;
%rr = (0.001*Rc:.0001:R0);
enju = bessel_deriv_zeros(M,K);
kperp = enju/R0;
rr = (R0*0.01:.0001:R0);
x = (0:.0001:4*enju);

% Bessel function - first kind M-th order
hold on;
temp_plus=besselj(M,x);
plot(x,temp_plus,'.-'); LegendDesc = {'J_{M}(r)'};
if(M>=1)
    temp_minus=besselj(M-1,x);
    plot(x,temp_minus,'.-'); LegendDesc{end+1,:} = 'J_{M-1}(r)';
end
temp_plus=besselj(M+1,x);
plot(x,temp_plus,'.-'); LegendDesc{end+1,:} = 'J_{M+1}(r)';
if(M>=2)
    temp_minus=besselj(M-2,x);
    plot(x,temp_minus,'.-'); LegendDesc{end+1,:} = 'J_{M-2}(r)';
end
temp_plus=besselj(M+2,x);
plot(x,temp_plus,'.-'); LegendDesc{end+1,:} = 'J_{M+2}(r)';
if(M>=3)
    temp_minus=besselj(M-3,x);
    plot(x,temp_minus,'.-'); LegendDesc{end+1,:} = 'J_{M-3}(r)';
end
temp_plus=besselj(M+3,x);
plot(x,temp_plus,'.-'); LegendDesc{end+1,:} = 'J_{M+3}(r)';
legend(LegendDesc);
title('Bessel function of first kind M-th order.');
%set(gca,'YColor','none');
set(gca,'YTick', []);
hold off;

% Bessel function derivative - first kind M-th order

figure();
hold on;
temp_plus=diff(besselj(M,x));
xx = x(1:end-1);
plot(xx,temp_plus,'.-'); LegendDesc = {'J_{M}(r)'};
if(M>=1)
    temp_minus=diff(besselj(M-1,x));
    plot(xx,temp_minus,'.-'); LegendDesc{end+1,:} = 'J_{M-1}(r)';
end
temp_plus=diff(besselj(M+1,x));
plot(xx,temp_plus,'.-'); LegendDesc{end+1,:} = 'J_{M+1}(r)';
if(M>=2)
    temp_minus=diff(besselj(M-2,x));
    plot(xx,temp_minus,'.-'); LegendDesc{end+1,:} = 'J_{M-2}(r)';
end
temp_plus=diff(besselj(M+2,x));
plot(xx,temp_plus,'.-'); LegendDesc{end+1,:} = 'J_{M+2}(r)';
if(M>=3)
    temp_minus=diff(besselj(M-3,x));
    plot(xx,temp_minus,'.-'); LegendDesc{end+1,:} = 'J_{M-3}(r)';
end
temp_plus=diff(besselj(M+3,x));
plot(xx,temp_plus,'.-'); LegendDesc{end+1,:} = 'J_{M+3}(r)';
legend(LegendDesc);
title('Bessel function of first kind M-th order derivative.');
%set(gca,'YColor','none');
set(gca,'YTick', []);
hold off;




%% Radial EM power distribution for first kind M-th order
figure();
hold on;

% Promien elektronow
%ElectronBeamRadiusPlus = bessel_deriv_zeros(SC.I.ModM+SC.I.HarmonicNumber,1)*(C.SpeedOfLight/SC.I.Frequency)/(2*pi);
%ElectronBeamRadiusMinus = bessel_deriv_zeros(SC.I.ModM-SC.I.HarmonicNumber,1)*(C.SpeedOfLight/SC.I.Frequency)/(2*pi);

% zero power at the wall - E=0 at metalic wall, from boundary condition
rrr = rr(1:end-1);
temp_plus=diff(besselj(M,bessel_deriv_zeros(M,K)*rr/R0)).^2;
plot(rrr,temp_plus/max(temp_plus),'.-'); LegendDesc = {'J_{M}(r)'};
if(M>=1)
    temp_minus=diff(besselj(M-1,bessel_deriv_zeros(M-1,K)*rr/R0)).^2;
    plot(rrr,temp_minus/max(temp_minus),'.-'); LegendDesc{end+1,:} = 'J_{M-1}(r)';
end
temp_plus=diff(besselj(M+1,bessel_deriv_zeros(M+1,K)*rr/R0)).^2;
plot(rrr,temp_plus/max(temp_plus),'.-'); LegendDesc{end+1,:} = 'J_{M+1}(r)';
if(M>=2)
    temp_minus=diff(besselj(M-2,bessel_deriv_zeros(M-2,K)*rr/R0)).^2;
    plot(rrr,temp_minus/max(temp_minus),'.-'); LegendDesc{end+1,:} = 'J_{M-2}(r)';
end
temp_plus=diff(besselj(M+2,bessel_deriv_zeros(M+2,K)*rr/R0)).^2;
plot(rrr,temp_plus/max(temp_plus),'.-'); LegendDesc{end+1,:} = 'J_{M+2}(r)';
if(M>=3)
    temp_minus=diff(besselj(M-3,bessel_deriv_zeros(M-3,K)*rr/R0)).^2;
    plot(rrr,temp_minus/max(temp_minus),'.-'); LegendDesc{end+1,:} = 'J_{M-3}(r)';
end
temp_plus=diff(besselj(M+3,bessel_deriv_zeros(M+3,K)*rr/R0)).^2;
plot(rrr,temp_plus/max(temp_plus),'.-'); LegendDesc{end+1,:} = 'J_{M+3}(r)';
legend(LegendDesc);
title('E-field power profile.');
ylabel("Normalized power [a.u.]")
xlabel("Radius [mm]")
hold off;


%% Resonator cross-section
B0 = 1;
freq = 24.5; %[GHz]
z = 0;
t = 1;

Theta = 0:2*pi/128:2*pi;
%fi = pi/2;
C = PhysicalConstants;

omega = 2*pi*freq;
enju = bessel_deriv_zeros(M,K); % characteristic value
kz=sqrt((omega/C.SpeedOfLight)^2-(enju/R0)^2);
hold on;
J = besselj(M,enju*rr/R0);
Jdiff = diff(J);

rr = (0.1:.1:R0);

[RR,TH] = ndgrid(rr,Theta);

Z = zeros(size(RR,1), size(RR,2));
J = besselj(M,enju.*RR/R0);




%% Movie loop
tfinish = floor(1/freq) + ceil( (1/freq-floor(1/freq))/0.001) * 0.001; %round up to 0.001 [ns]
tstep = floor(1/freq/100) + ceil( (1/freq/100-floor(1/freq/100))/0.0001) * 0.0001; % round to 0.0001 [ns]
time = 0:tstep:tfinish; %[ns]
Folder = './wyniki/'; 

hash = datestr(now,30);
FolderName=sprintf('%s%s', Folder, hash);
mkdir(FolderName);
licznik=1;
for t=time
    close all;
    clearvars ZErr ZEth ZE;
    ZErr = zeros(size(RR,1) , size(RR,2));
    ZEth = zeros(size(RR,1) , size(RR,2));
    
    for row=1:size(RR,1) 

        for col=1:size(RR,2)
           r = RR(:, col);
           th = TH(1, col);
           ZErr(:,col) = -B0*M*omega*R0*(1/enju)^2*(R0./r) .* besselj(M,enju*r/R0) * sin(M*th + kz*z - omega*t);

           tmp = B0*omega*R0*(1/enju)*R0*diff(besselj(M,enju*r/R0)) * cos(M*th + kz*z - omega*t);
           tmp = [tmp; tmp(end)]; % adding one element to match matrixes length
           ZEth(:,col) = tmp;

        end
    end

    figHandle = figure('units','centimeters','position',[10,10,30,25]);
    subplot(2,2,1);
    ZE = sqrt(ZEth.^2 + ZErr.^2);
    h1 = surf(RR.*cos(TH), RR.*sin(TH),ZEth.^2); % Matlab plots in kartesian space
    title('E-field angular component power.');
    zlabel("Power [a.u]")
    set(gca,'XColor', 'none','YColor','none')
 %   h1.EdgeColor = 'none';
    
    subplot(2,2,3);
    h2 = surf(RR.*cos(TH), RR.*sin(TH),ZErr.^2);
    title('E-field radial component power.');
    zlabel("Power [a.u]")
    set(gca,'XColor', 'none','YColor','none')
 %   h2.EdgeColor = 'none';

    subplot(2,2,2);
    colormap(jet);
    h3 = surf(RR.*cos(TH), RR.*sin(TH),ZE.^2./max(ZE.^2,[],'all')); % Normalized to one
    title('E-field total power distribution.');
    zlabel("Power [a.u]")
    set(gca,'XColor', 'none','YColor','none')
 %   h3.EdgeColor = 'none';
    
    subplot(2,2,4);
    colormap(jet);
    h4 = contourf(RR.*cos(TH), RR.*sin(TH),ZE.^2./max(ZE.^2,[],'all')); % Normalized to one

    colorbar;
    set(gca,'visible','off')

    PlotTitle = sprintf("Mode TE_{%d, %d} at freq= %.2f [GHz], R_0= %.2f [mm], t= %.4f [ns] and z = %.2f [mm]", M, K, freq, R0, t, z);
    sgtitle(PlotTitle);

    saveas(figHandle, sprintf('%s/time%06d.bmp',  FolderName, licznik), 'bmp'); % No commas for virtualdub
    licznik=licznik+1;
end

% %% Filmik z narostem pola E
% if(TD.Movie == 1)
%     mkdir(sprintf('%s/%s', FolderName, 'filmik'));
%  
%     for k=1:size(abs(TD.O.RFfieldInTime(:,:)),2)
%         FigHandle = figure();
%         
%         
%         plot(TD.I.RFfieldAxisZ,abs(TD.O.RFfieldInTime(:,k)));
%         xlim([0 max(TD.I.RFfieldAxisZ)]);
%         ylim([0 max(max(abs(TD.O.RFfieldInTime(:,:))))*1.1]);
%         xlabel('Z axis [mm]', 'fontsize', 12);
%                
%         ylabel(sprintf('Rf Field [a.u], Pout=%0.1f [kW]' , TD.O.GyrotronOutputPower(k)), 'fontsize', 12);        
%         title(sprintf('Hash: %s, czas: %.2f [ns], Uc=%0.1f [kV]' ,hash, TD.I.TimeSpan(k), TD.O.Ucathode(k)));
%         
%         %inset
%         InsetHandle = figure();
%         plot(TD.O.TimeSpan(k),TD.O.GyrotronOutputPower(k),'r.','MarkerSize',20);hold on;
%         plot(TD.O.TimeSpan,TD.O.GyrotronOutputPower);set(gca,'FontSize',6); hold off;
%         xlabel('Time [ns]', 'fontsize', 6); ylabel('Output Power [kW]', 'fontsize', 6);
%         
%         [~, NewAxHandle, NewFigHandle]=inset(FigHandle,InsetHandle);
%         
%         set(gcf, 'paperunits', 'centimeters', 'paperposition', [0 0 19.5 11]);
%         set(NewFigHandle, 'InvertHardcopy', 'off');
%         
%         saveas(NewFigHandle, sprintf('%s/filmik/%06.2f.bmp',  FolderName, TD.I.TimeSpan(k)), 'bmp');
%         
%         %saveas(NewFigHandle, sprintf('test.bmp',  FolderName, TD.I.TimeSpan(k)), 'bmp');
%         close(FigHandle); close(InsetHandle); close(NewFigHandle); 
%         
%     end
% end

