clear all; close all;
%% Project assumptions - input data

CC.I.ModM = 0;  % Mod TEmk1
CC.I.ModK = 1;  % Mod TEmk1    
CC.I.ResonatorRadiusR0 = 7.5; %[mm]
CC.I.HarmonicNumber = 1;
% [degree] angle between resonator parts
CC.I.ResonatorAngle = [2.3;0.0;3.00]; 
% [mm] resonator length, input tapper, middle part, output tapper
CC.I.ResonatorLength = [30.0; 75; 30.0];;
CC.I.StepZ =0.05; % Z axis step [mm]  must be smaller than: c/f * (1/20) s. 49 Edgecomb

%% Constants
C = PhysicalConstants;
CC.Comment = 'Cold Cavity';
CC.I.Comment = 'Cold Cavity Input';

%% Cold Cavity Approach
DimmensionSpan = (0.70*CC.I.ResonatorLength(2):0.1:1.3*CC.I.ResonatorLength(2));


%Prepare data for parallel computing
for m = 1:length(DimmensionSpan)
    CC.I.ResonatorLength = [CC.I.ResonatorLength(1); DimmensionSpan(m); CC.I.ResonatorLength(3)];
    ParforIn(m) = CC.I;
end
%Run parallel optimization for different cavity length L2
parfor m = 1:length(DimmensionSpan) 
   ParforOut(m) = ColdCavityApproachFunction(ParforIn(m), C, 0);
end
i = 1;



SmallestBoundaryErrorVal = 99999;
SmallestBoundaryErrorPosition = 99999;
for m = 1:length(DimmensionSpan)
    pks = findpeaks(abs(ParforOut(m).RFfieldNormalized), ParforOut(m).RFfieldAxisZ, 'MinPeakWidth', ParforIn(m).StepZ*100); 
    ParforOut(m).ResonatorCentralLengthMin = DimmensionSpan(1);
    ParforOut(m).ResonatorCentralLengthMax = DimmensionSpan(end);
    ParforOut(m).NumberOfPeaks = length(pks);
    %if ((ParforOut(m).NumberOfPeaks <= CC.I.HarmonicNumber+1) && (max(abs(ParforOut(m).RFfieldNormalized)) > abs(ParforOut(m).RFfieldNormalized(end))) )
    
        if(ParforOut(m).BoundaryConditionErrorOptimal < SmallestBoundaryErrorVal)
            SmallestBoundaryErrorVal = ParforOut(m).BoundaryConditionErrorOptimal;
            SmallestBoundaryErrorPosition = m;
            CC.O.GoodSolutions{i} = ParforOut(m); i=i+1;
        end
   % end
    CC.GoodSolutions{m} = ParforOut(m);
end
CC.O = ParforOut(SmallestBoundaryErrorPosition);  % Best solution

clearvars -except C CC ParforOut;

% Stara wersja
%CC.O = ColdCavityApproachFunction(CC.I, C, 1);



%% Self Consisten Approach - wyliczenie sprawności

%BiggestMaxEfficiencyValue = 0.10; % poniej 10 % sprawnoci nie zapamitujemy
BiggestMaxEfficiencyValue = 0.001; % chwilowo zeby poszlo dla 1 rozwiazania

i = 1;
for m = 1:length(ParforOut)
    
    clearvars ColdCavityOut
    ColdCavityOut = ParforOut(m);

    SC.I.Alpha = 1.4;  % pitch factor
    SC.I.Ucathode = 12; % [kV] cathode voltage
    %SC.I.MagneticFieldB = 0.87878 ; % magnetic field in the resonator [T]
    SC.I.GyrotronOutputPower = 1; % [kW] assumed output power
    SC.I.NumberOfElectrons = 24;  

    % Take ColdCavityApproach data
    SC.I.Frequency = ColdCavityOut.Foptimal;
    SC.I.Qoptimal = ColdCavityOut.Qoptimal;  % Qdiffractive optimal
    SC.I.Qohm = ColdCavityOut.Qohm;
    SC.I.Qtotal = ColdCavityOut.Qtotal;  %% Qtotal - Qohmic
    SC.I.RFfieldRaw = ColdCavityOut.RFfieldRaw;
    SC.I.RFfieldAxisZ = ColdCavityOut.RFfieldAxisZ;
    SC.I.CharacteristicValue = ColdCavityOut.CharacteristicValue;
    SC.I.HarmonicNumber = ColdCavityOut.HarmonicNumber;
    SC.I.ModM = ColdCavityOut.ModM;
    SC.I.ModK = ColdCavityOut.ModK;
    SC.I.ResonatorRadiusR0 = ColdCavityOut.ResonatorRadiusR0;
    SC.I.ResonatorRadius = ColdCavityOut.ResonatorRadius;

    %SC.O = SelfConsistentApproachFunction(SC.I, C, 1, 0);
    ForOut(m) = OptymizeMagneticFieldFunction(SC.I, C, 0);
    ForOutResonator(m).ResonatorLength = ColdCavityOut.ResonatorLength; % ParforOut(m).ResonatorLength
    ForOutResonator(m).ResonatorAngle = ColdCavityOut.ResonatorAngle;
    ForOutResonator(m).ModM = ColdCavityOut.ModM;
    ForOutResonator(m).ModK = ColdCavityOut.ModK;
    ForOutResonator(m).Foptimal = ColdCavityOut.Foptimal;
    ForOutResonator(m).Qoptimal = ColdCavityOut.Qoptimal;
    ForOutResonator(m).Qtotal = ColdCavityOut.Qtotal;
    ForOutResonator(m).BoundaryConditionErrorOptimal = ColdCavityOut.BoundaryConditionErrorOptimal;
    ForOutResonator(m).ResonatorRadiusR0 = ColdCavityOut.ResonatorRadiusR0;
    
    if(ForOut(m).MaxEfficiencyValue > BiggestMaxEfficiencyValue)
            BiggestMaxEfficiencyValue = ForOut(m).MaxEfficiencyValue;
            BiggestMaxEfficiencyPosition = m; ForOut(m).MaxEfficiencyValue
            i=i+1;
    end
    
    SC.Solutions{m} = ForOut(m);
    %SC.O = OptymizeAlphaFunction(SC.I, C, 1);

end
SC.O = ForOut(BiggestMaxEfficiencyPosition);  % Best solution
SC.O.ColdCavityRFfieldRaw = CC.GoodSolutions{1, BiggestMaxEfficiencyPosition}.RFfieldRaw;
SC.O.AxisZ = CC.GoodSolutions{1, BiggestMaxEfficiencyPosition}.RFfieldAxisZ;

SC.O.ColdCavityRFfieldRaw = CC.GoodSolutions{1, BiggestMaxEfficiencyPosition}.RFfieldRaw;
SC.O.ColdCavity.Qmin = CC.GoodSolutions{1, BiggestMaxEfficiencyPosition}.Qmin;
SC.O.ColdCavity.Qmax = CC.GoodSolutions{1, BiggestMaxEfficiencyPosition}.Qmax;
SC.O.ColdCavity.Fmin = CC.GoodSolutions{1, BiggestMaxEfficiencyPosition}.Fmin;
SC.O.ColdCavity.Fmax = CC.GoodSolutions{1, BiggestMaxEfficiencyPosition}.Fmax;
SC.O.ColdCavity.ResonatorCentralLengthMin = CC.GoodSolutions{1, BiggestMaxEfficiencyPosition}.ResonatorCentralLengthMin;
SC.O.ColdCavity.ResonatorCentralLengthMax = CC.GoodSolutions{1, BiggestMaxEfficiencyPosition}.ResonatorCentralLengthMax;

%% Zapis wynikw

% Kasujemy pozostae rozwizania przed zapisem zeby oszczdzi miejsce na
% dysku
SC.Solutions = [];
SC.O.SOLUTIONSforB = [];
CC.GoodSolutions = [];

plot(SC.O.AxisZ, abs(SC.O.AllEletronMomentum));
plot( SC.O.AxisZ, abs(SC.O.ColdCavityRFfieldRaw));

%% Zapis wynikow do pliku
%Folder = '../../wyniki/';
Folder = 'wyniki/';

hash = datestr(now,30);
FolderName=sprintf('%s%s', Folder, hash);
mkdir(FolderName);
fileParameters = sprintf('%s/%s', FolderName, 'parameters.txt');
fileRFfieldFig = sprintf('%s/%s', FolderName, 'RFfield');
fileElectronMomentumFig = sprintf('%s/%s', FolderName, 'ElectronMomentum');
fileResults = sprintf('%s/%s', FolderName, 'results.mat');

fileHandle = fopen ( fileParameters , 'wt' );
dane=sprintf('=============================== The most efficient solution %s : ===============================', hash);disp(dane); fprintf ( fileHandle , '%s\n' , dane );
dane=sprintf('Efficiency = %.4f, for Length=[ %.2f ; %.2f ; %.2f ], R_{0}= %.2f Angle: [ %.2f; %.2f; %.2f ], Mod M = %i, Mod K = %i', SC.O.MaxEfficiencyValue , ForOutResonator(BiggestMaxEfficiencyPosition).ResonatorLength, ForOutResonator(BiggestMaxEfficiencyPosition).ResonatorRadiusR0, ForOutResonator(BiggestMaxEfficiencyPosition).ResonatorAngle, ForOutResonator(BiggestMaxEfficiencyPosition).ModM, ForOutResonator(BiggestMaxEfficiencyPosition).ModK);disp(dane); fprintf ( fileHandle , '%s\n' , dane );
dane=sprintf('B_{opt} = %.5f, U_{cath} = %.2f, I_{cath} = %.2f Power_{out} = %.2f Alpha = %.2f Freq_{opt} = %.5f, Q_{opt} = %.5f, Q_{total} = %.5f ', ForOut(BiggestMaxEfficiencyPosition).MagneticFieldB ,ForOut(BiggestMaxEfficiencyPosition).Ucathode, ForOut(BiggestMaxEfficiencyPosition).Icathode, ForOut(BiggestMaxEfficiencyPosition).GyrotronOutputPower, ForOut(BiggestMaxEfficiencyPosition).Alpha, ForOutResonator(BiggestMaxEfficiencyPosition).Foptimal, ForOutResonator(BiggestMaxEfficiencyPosition).Qoptimal, ForOutResonator(BiggestMaxEfficiencyPosition).Qtotal);disp(dane); fprintf ( fileHandle , '%s\n' , dane );
dane=sprintf('Q_range = [ %.1f - %.1f ], Freq_range = [ %.1f - %.1f ], Lcentral_range = [ %.3f - %.3f ]', SC.O.ColdCavity.Qmin, SC.O.ColdCavity.Qmax, SC.O.ColdCavity.Fmin, SC.O.ColdCavity.Fmax, SC.O.ColdCavity.ResonatorCentralLengthMin, SC.O.ColdCavity.ResonatorCentralLengthMax);disp(dane); fprintf ( fileHandle , '%s\n' , dane );

fclose(fileHandle);

save(fileResults, 'CC', 'SC');

%% Rysowanie wykresow mocy wyjściowej

figHandle = figure();
plot(SC.O.AxisZ, abs(SC.O.AllEletronMomentum));
str=sprintf('Efficiency = %.4f, for Length=[ %.2f ; %.2f ; %.2f ], R_{0}= %.2f Angle: [ %.2f; %.2f; %.2f ], Mod M = %i, Mod K = %i', SC.O.MaxEfficiencyValue , ForOutResonator(BiggestMaxEfficiencyPosition).ResonatorLength, ForOutResonator(BiggestMaxEfficiencyPosition).ResonatorRadiusR0, ForOutResonator(BiggestMaxEfficiencyPosition).ResonatorAngle, ForOutResonator(BiggestMaxEfficiencyPosition).ModM, ForOutResonator(BiggestMaxEfficiencyPosition).ModK);
legend('Electron momentum',str, 'Location', 'NorthWest');
set(gcf(),'Name','Electron momentum');
xlabel('Axis Z [mm]'); ylabel('Electron momentum');
title(sprintf('Hash: %s',hash));
savefig(figHandle, fileElectronMomentumFig);

figHandle = figure();
plot( SC.O.AxisZ, abs(SC.O.ColdCavityRFfieldRaw));
str=sprintf('Efficiency = %.4f, for Length=[ %.2f ; %.2f ; %.2f ], R_{0}= %.2f Angle: [ %.2f; %.2f; %.2f ], Mod M = %i, Mod K = %i', SC.O.MaxEfficiencyValue , ForOutResonator(BiggestMaxEfficiencyPosition).ResonatorLength, ForOutResonator(BiggestMaxEfficiencyPosition).ResonatorRadiusR0, ForOutResonator(BiggestMaxEfficiencyPosition).ResonatorAngle, ForOutResonator(BiggestMaxEfficiencyPosition).ModM, ForOutResonator(BiggestMaxEfficiencyPosition).ModK);
legend('RFfield [a.u.]',str, 'Location', 'NorthWest');
set(gcf(),'Name','RFfield [a.u.]');
xlabel('Axis Z [mm]'); ylabel('RFfield [a.u.]');
title(sprintf('Hash: %s',hash));
savefig(figHandle, fileRFfieldFig);

 


