clear all; close all;
%% Project assumptions - input data

CC.I.ModM = 0;  % Mod TEmk1
CC.I.ModK = 1;  % Mod TEmk1    
CC.I.ResonatorRadiusR0 = 7.5; %[mm]
CC.I.HarmonicNumber = 1;
% [degree] angle between resonator parts
CC.I.ResonatorAngle = [2.3;0.0;3.00]; 
% [mm] resonator length, input tapper, middle part, output tapper
CC.I.ResonatorLength = [30.0; 70.9; 30.0]; 
CC.I.StepZ =0.05; % Z axis step [mm]  must be smaller than: c/f * (1/20) s. 49 Edgecomb

%% Constants
C = PhysicalConstants;
CC.Comment = 'Cold Cavity';
CC.I.Comment = 'Cold Cavity Input';

%% Cold Cavity Approach
%CC.I.Qspan = 50; % Qspan accuracy - for Full Review it must be less accurate to safe time
CC.O = ColdCavityApproachFunction(CC.I, C, 1); % 0 - no pots, 1 do plots



%% Store results to a file % folder results has to be created manualy
Folder = 'results/';

hash = datestr(now,30);
FolderName=sprintf('%s%s', Folder, hash);
mkdir(FolderName);
fileParameters = sprintf('%s/%s', FolderName, 'parameters.txt');
fileRFfieldFig = sprintf('%s/%s', FolderName, 'RFfield');

fileHandle = fopen ( fileParameters , 'wt' );
dane=sprintf('=============================== COLD CAVITY PART ===============================');disp(dane); fprintf ( fileHandle , '%s\n' , dane );
dane=sprintf('TE: M=%d K=%d P=%d HarmonicNumber=%d ResonatorRadiusR0=%.15g [mm]',CC.I.ModM, CC.I.ModK, 1, CC.I.HarmonicNumber, CC.I.ResonatorRadiusR0);disp(dane); fprintf ( fileHandle , '%s\n' , dane );
dane=sprintf('ResonatorAngle = %.15g; %.15g; %.15g [deg]\nResonatorLength = %.15g; %.15g; %.15g StepZ = %.15g [mm]',CC.I.ResonatorAngle, CC.I.ResonatorLength, CC.I.StepZ);disp(dane); fprintf ( fileHandle , '%s\n' , dane );
dane=sprintf('Fspan = %.10g: %.10g: %.10g Foptimal= %.10g [GHz]',CC.O.Fmin, (CC.O.Fmax- CC.O.Fmin), CC.O.Fmax, CC.O.Foptimal);disp(dane); fprintf ( fileHandle , '%s\n' , dane );
dane=sprintf('Qspan = %.10g: %.10g: %.10g QOptimal = %.10g , Qohm = %.10g , Qtotal = %.10g [a.u.]',CC.O.Qmin, (CC.O.Qmax- CC.O.Qmin), CC.O.Qmax, CC.O.Qoptimal, CC.O.Qohm, CC.O.Qtotal);disp(dane); fprintf ( fileHandle , '%s\n' , dane );
dane=sprintf('CharacteristicValue = %.15g',CC.O.CharacteristicValue);disp(dane); fprintf ( fileHandle , '%s\n' , dane );
fclose(fileHandle);

figHandle = figure('units','centimeters','position',[10,10,25,15]);
colormap(jet);
plot(CC.O.RFfieldAxisZ,real(CC.O.RFfieldNormalized)*CC.I.ResonatorRadiusR0,CC.O.RFfieldAxisZ, imag(CC.O.RFfieldNormalized)*CC.I.ResonatorRadiusR0,CC.O.RFfieldAxisZ, abs(CC.O.RFfieldNormalized)*CC.I.ResonatorRadiusR0, CC.O.RFfieldAxisZ,CC.O.ResonatorRadius,'LineWidth',2.0);
set(gcf(),'Name','fi(z)');xlabel('z [mm]'); ylabel('f(z)');
legend('Re{RFfield}','Im{RFfield}','|RFfield|', 'R(z)');
title(sprintf('Hash: %s',hash));

savefig(figHandle, fileRFfieldFig); % save image to results folder
saveas(figHandle, sprintf('%s.png',  fileRFfieldFig), 'png');  % uncomment
% to generate bitmap high quality image
 